<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 5/9/17
 * Time: 8:56 AM
 */

namespace qfq;

require_once(__DIR__ . '/../exceptions/CodeException.php');
require_once(__DIR__ . '/Database.php');

/*
 * Read the extension version number.
 * Read the QFQ database version number: stored in the comment field of table 'Form'. Format:  Version=x.y.z
 * If versions different:
 *   * Read the update array 'DatabaseUpdateData.php'.
 *   * Play all changes from the update array after 'old' upto 'new'.
 *   * Save new QFQ database version in the comment field of table 'Form'
 *
 * In a new QFQ installation, the comment field of table 'Form' is empty. On the first call of QFQ, the version string
 * will be set. Also the 'formEditor.sql' will be played initially.
 *
 */

class DatabaseUpdate {

    /**
     * @var Database instantiated class
     */
    protected $db = null;

    /**
     * @param Database $db
     */
    public function __construct(Database $db) {
        $this->db = $db;
    }

    /**
     * @return mixed
     * @throws CodeException
     */
    private function getExtensionVersion() {
        $path = __DIR__ . '/../../../ext_emconf.php';
        $_EXTKEY = EXT_KEY;
        $EM_CONF = NULL;
        if (@file_exists($path)) {
            include $path;

            if (isset($EM_CONF[$_EXTKEY]['version'])) {
                return $EM_CONF[$_EXTKEY]['version'];
            }
        }

        throw new CodeException('Failed to read extension version', ERROR_QFQ_VERSION);
    }

    /**
     * @return bool|string  false if there is no table 'Form' or if there is no comment set in table 'Form'.
     * @throws CodeException
     * @throws DbException
     */
    private function getDatabaseVersion() {
        $row = $this->db->sql("SHOW TABLE STATUS WHERE Name='Form'", ROW_EXPECT_0_1);

        if (isset($row['Comment'])) {
            $arr = explode('=', $row['Comment']);
            if (count($arr) != 2 || $arr[0] != 'Version' || $arr[1] == '') {
                return false;
            }
        } else {
            return false;
        }

        return $arr[1];
    }

    /**
     * @param $version
     * @throws CodeException
     * @throws DbException
     */
    private function setDatabaseVersion($version) {

        $this->db->sql("ALTER TABLE `Form` COMMENT = 'Version=$version'");

    }

    /**
     * @throws CodeException
     */
    public function checkNupdate() {

        $new = $this->getExtensionVersion();
        $old = $this->getDatabaseVersion();

        if ($new != $old) {
            $this->dbUpdateStatements($old, $new);
            $this->db->playSqlFile(__DIR__ . '/../../sql/formEditor.sql');
            $this->setDatabaseVersion($new);
        }
    }

    /**
     * @param $path
     * @return array
     * @throws CodeException
     */
    private function readUpdateData($path) {

        if (!@file_exists($path)) {
            throw new CodeException("File '$path'' not found", ERROR_IO_OPEN);
        }

        $UPDATE_ARRAY = null;

        include $path;

        return $UPDATE_ARRAY;

    }

    /**
     * Play all update statement with version number are '>' than $old and '<=' to $new.
     *
     * @param $old
     * @param $new
     */
    private function dbUpdateStatements($old, $new) {
        if ($new == '' || $old === false) {
            return;
        }
        $updateArray = $this->readUpdateData(__DIR__ . '/DatabaseUpdateData.php');

        $apply = false;
        foreach ($updateArray as $key => $sqlStatements) {

            // Search starting point to apply updates
            $rc = version_compare($key, $old);
            if ($rc == 1) {
                $apply = true;
            }

            if ($apply) {
                // Play Statements
                foreach ($sqlStatements as $sql) {
                    $this->db->sql($sql);
                }
            }
        }
    }
}